@@ -2,7 +2,7 @@ X ("inhibit-splash-screen", "nil", false, "Non-nil inhibits the startup screen.\
 X ("standard-indent", "4", false, "Default number of columns for margin-changing functions to indent.")
 X ("tab-width", "8", true, "Distance between tab stops (for display of tab characters), in columns.")
 X ("tab-always-indent", "t", false, "Controls the operation of the TAB key.\nIf t, hitting TAB always just indents the current line.\nIf nil, hitting TAB indents the current line if point is at the\nleft margin or in the line's indentation, otherwise it inserts a\n\"real\" TAB character.")
-X ("indent-tabs-mode", "nil", true, "If non-nil, insert-tab inserts \"real\" tabs; otherwise, it always inserts\nspaces.")
+X ("indent-tabs-mode", "t", true, "If non-nil, insert-tab inserts \"real\" tabs; otherwise, it always inserts\nspaces.")
 X ("fill-column", "70", true, "Column beyond which automatic line-wrapping should happen.\nAutomatically becomes buffer-local when set in any fashion.")
 X ("auto-fill-mode", "nil", false, "If non-nil, Auto Fill Mode is automatically enabled.")
 X ("kill-whole-line", "nil", false, "If non-nil, `kill-line' with no arg at beg of line kills the whole line.")
